/**
 * 
 */
package org.lantsovanton.abstraction.model.impl;

import java.util.*;
import java.util.ListIterator;

import org.lantsovanton.abstraction.model.IChip;
import org.lantsovanton.abstraction.model.IListener;
import org.lantsovanton.abstraction.model.IPosition;


/**
 * @author 
 *
 */
public class Position implements IPosition {
	
	
	private final int myHeight;
	private final int myWidth;
	private IChip[][]  myBoard;
	private int myNextMovePlayer;	
	private LinkedList<IListener> myList = new LinkedList<IListener>(); 
	
	public Position(int h, int l, int player ){
		myHeight = h;
		myWidth = l;
		myBoard = new Chip[h][l];
		myNextMovePlayer = player;
	}
	
	private void update(){
		ListIterator<IListener> it = myList.listIterator();
		while( it.hasNext() ){
			it.next().update();
		}
	}
	
	public int getPlayer(){
		return myNextMovePlayer;
	}	
	
	public IChip get( int h, int l ){
		return myBoard[h][l];
	}
	
	public int getHight(){
		return myHeight;
	}
	
	public int getWidth(){
		return myWidth;
	}
	
	public void set( int h, int l, IChip fishka  ){
		myBoard[h][l] = fishka;
		update();
	}
	
	public void setPlayer( int player ){
		myNextMovePlayer = player;
		update();
	}
		
	public String toString(){
		String text = new String();
		text += "\n";
		text += this.myNextMovePlayer + " ";
		text += this.myHeight + " ";
		text += this.myWidth + " " + "\n" ; 
		for( int i = 0; i < myHeight; i++ ){
			for( int j = 0; j < myWidth; j++ ){
				if( myBoard[i][j] == null ){
					text += 0+" ";
					continue;
				}
				text += myBoard[i][j].getPlayer()+" ";
			}
			text += "\n";
		}
		return text;
	}
	
	public Object clone(){
		Position result = new Position( myHeight ,myWidth, myNextMovePlayer );
		for( int h = 0; h < myHeight; h++ ){
			for( int l = 0; l < myWidth; l++ ){
				result.set( h, l, this.get(h,l) );
			}
		}
		
		return result;
	}	

	public void addListener(IListener listener) {
		myList.addLast( listener );		
	}

	public void removeListener(IListener listener) {
		myList.remove( listener );
		
	}
	
	public boolean equals( Object o ){
		if( !( o instanceof IPosition  ) ){
			return false;
		}
		IPosition position = (IPosition)o;
		if( position.getHight() != this.getHight() || position.getWidth() != this.getWidth()  ){
			return false;
		}
		if( position.getPlayer() != this.getPlayer() ){
			return false;
		}
		for( int i = 0; i < this.getHight(); i++ ){
			for( int j = 0; j < this.getWidth(); j++ ){
				if( this.get( i, j ) != position.get( i, j )   ){
					return false;
				}
			}
		}		
		return true;
	}

}
