package org.lantsovanton.gameio;

import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.lantsovanton.abstraction.model.Factory;
import org.lantsovanton.abstraction.model.IGame;
import org.lantsovanton.abstraction.model.IPosition;
import org.lantsovanton.abstraction.model.Intellect;
//import org.lantsovanton.abstraction.model.impl.Chip;
//import org.lantsovanton.abstraction.model.impl.Position;
import org.lantsovanton.games.GameFactory;


import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class XMLGameReader implements IGameReader {
	private File myFile;
	private IPosition myPosition;
	private Document myDocument;
	private int myHeight;
	private int myWidth;
	private int myPlayer;
	private Intellect myIntellect;
	private IGame myGame;
	private String myGameName;
	
	private Factory myFactory =new Factory();
    private boolean myBeautiful = true; 
	
	public File getFile(){
		return myFile;
	}
	public XMLGameReader(){
		
	}
	
	public void setFile( File file ){
		myFile = file;
		try{
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = factory.newDocumentBuilder();		
			myDocument = builder.parse(file);
		}catch (SAXException e) {
			System.out.println("Cannot parse file \"" + file + "\": " + e.getMessage() + e.getStackTrace() );
			return;
		} catch (IOException e) {
			System.out.println("Cannot read file \"" + file + "\": " + e.getMessage());
			return;
		} catch (ParserConfigurationException e) {
			System.out.println("Cannot create parser \"" + file + "\": " + e.getMessage());
			return;
		}
		Element root = myDocument.getDocumentElement();
		
		myPlayer = this.getValueOfNode( root.getAttributes().getNamedItem("startplayer") );
		myWidth = this.getValueOfNode( root.getAttributes().getNamedItem("width") );
		myHeight = this.getValueOfNode(root.getAttributes().getNamedItem("height") );
		myGameName =  root.getAttributes().getNamedItem("game").getNodeValue();
		try{
			myBeautiful =  root.getAttributes().getNamedItem("beautiful").getNodeValue().endsWith("true");
		}catch( Throwable e ){
			
		}
		
		myPosition = myFactory.initialPosition( myHeight, myWidth, myPlayer );
		try {
			new GameFactory();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	
		myGame = GameFactory.newInstace( myGameName );
		myIntellect = myGame.initialIntellect();
		NodeList counters = root.getElementsByTagName("chip");
		for( int i = 0; i < counters.getLength(); i++ ){
			Node counter = counters.item( i );
			this.setCounter( counter );
		}		
		
	}
	
	private void setCounter( Node node ){
		
		int player ;
		int x;
		int y;
		player = this.getValueOfNode( node.getAttributes().getNamedItem("player") );
		x= this.getValueOfNode( node.getAttributes().getNamedItem("x") );
		y = this.getValueOfNode( node.getAttributes().getNamedItem("y") );
		
		myPosition.set( y, x, myFactory.initialChip( player ) );
		
		
	}
	
	private int getValueOfNode(Node node){
		return Integer.parseInt( node.getNodeValue() );
		
	}
	
	public IPosition loadPosition(){
		return ( IPosition )myPosition.clone();
	}
	public IGame loadGame() {
		
		return myGame;
	}
	public Intellect loadIntellect() {
		
		return myIntellect;
	}
	public boolean loadbeautiful() {
		
		return myBeautiful;
	}

}
