package org.lantsovanton.games.reversi;

import java.util.Random;

import org.lantsovanton.GameException;
import org.lantsovanton.abstraction.model.*;
import org.lantsovanton.abstraction.model.impl.Move;





public class RandomReversi implements Intellect {
	 private IGame myGame;
	 private int myPlayer;
	 private IPosition myPosition;
	 private String myName = "RandomReversi";
	 private boolean myPermit;
	 
	 
	 public RandomReversi(){
		 myPermit = true;
	 }
	 
	 public RandomReversi( IGame game, int player, IPosition position ){
		 myGame = game;
		 myPlayer = player;
		 myPosition = position;
	 }
	 
	 /** 
	 * @see org.lantsovanton.abstraction.model.Intellect#getGame()
	 */
	public IGame getGame() {
		return myGame;
	}
	/** 
	 * @see org.lantsovanton.intellect.Intellect#getPlayer()
	 */
	public int getPlayer() {		
		return myPlayer;
	}
	/** 
     * @see org.lantsovanton.intellect.Intellect#getPosition()
	 */
	public IPosition getPosition() {		
		return myPosition;
	}

	public IMove go() {
		Random random = new Random(); 
		if( myPosition.getPlayer() != myPlayer || myGame.isGameFinish( myPosition ) ){
			throw new GameException();
		}
		
		for( int i = 0; i < 8; i += 7 ){
			for( int j = 0; j < 8; j += 7 ){
				IMove move = new Move( i, j, myPlayer );
				if( myGame.isMovePosible( myPosition, move ) ){
					return move;
				}
			}
		}
		
		for( int i = 0; i < 8; i += 7 ){
			for( int j = 1; j < 7; j++ ){
				IMove move1 = new Move( i, j, myPlayer );
				IMove move2 = new Move( j, i, myPlayer );
				if( myGame.isMovePosible( myPosition, move1 ) ){
					IPosition position2 = myGame.tryMove( myPosition, move1 );
					boolean flag1 = false; 
					boolean flag2 = false; 
					boolean flag3 = false;
					if( position2.get(i,j+1) != null ){
						if( position2.get(i,j+1).getPlayer() != myPlayer ){
							flag1 =true;
						}
					}
					if( position2.get(i,j-1) != null ){
						if( position2.get(i,j-1).getPlayer() != myPlayer ){
							flag2 =true;
						}
					}
					
					for( int k = 0 ;k < 8; k++ ){
						for( int l = 0; l < 8; l++ ){
							if( ( l==0 || l==7 || k==7 || k==0 ) && ( l != k ) && ( l+k != 7 ) ){
								if( position2.get( k, l ) == null ){
									if( l==0 || l==7  ){
										
										if( position2.get( k+1, l ) != null &&  position2.get( k-1, l ) != null ){
											if( position2.get( k+1, l ).getPlayer() == myPlayer  &&  position2.get( k-1, l ).getPlayer() == myPlayer ){
												flag3 = true;
												
											}
										}
										
									}else{	
										
										if( position2.get( k, l+1 ) != null &&  position2.get( k, l-1 ) != null ){
											if( position2.get( k, l+1 ).getPlayer() == myPlayer  &&  position2.get( k, l-1 ).getPlayer() == myPlayer ){
												flag3 = true;
												
											}
										}
										
																												
									}								
								}
												
							}
						}
					}
					
					for( int k = 0 ;k < 8; k++ ){
						for( int l = 0; l < 8; l++ ){
							if( ( l==0 || l==8 || k==8 || k==0 ) && ( l != k || l+k != 7 ) ){
								if( position2.get( k, l ) == null ){
									if( (l==0 || l==7) && k!=7 && k!=0 ){
										
										if( position2.get( k+1, l ) != null &&  position2.get( k-1, l ) != null ){
											if( position2.get( k+1, l ).getPlayer() == myPlayer  &&  position2.get( k-1, l ).getPlayer() == myPlayer ){
												flag3 = true;
												
											}
										}
										
									}else{	
										if( ( k==0 || k==7  ) && l!=0 && l!=7  )
										if( position2.get( k, l+1 ) != null &&  position2.get( k, l-1 ) != null ){
											if( position2.get( k, l+1 ).getPlayer() == myPlayer  &&  position2.get( k, l-1 ).getPlayer() == myPlayer ){
												flag3 = true;
												
											}
										}
										
																												
									}								
								}
												
							}
						}
					}
					
					if( ( ( !flag1 && !flag2 ) ||  ( flag1 && flag2 ) ) && ! flag3  )
					return move1;
				}
				
				if( myGame.isMovePosible( myPosition, move2 ) ){
					IPosition position2 = myGame.tryMove( myPosition, move2 );
					boolean flag1 = false; 
					boolean flag2 = false; 
					boolean flag3 = false;
					if( position2.get(j+1,i) != null ){
						if( position2.get(j+1,i).getPlayer() != myPlayer ){
							flag1 =true;
						}
					}
					if( position2.get(j-1,i) != null ){
						if( position2.get(j-1,i).getPlayer() != myPlayer ){
							flag2 =true;
						}
					}
					
					for( int k = 0 ;k < 8; k++ ){
						for( int l = 0; l < 8; l++ ){
							if( ( l==0 || l==8 || k==8 || k==0 ) && ( l != k || l+k != 7 ) ){
								if( position2.get( k, l ) == null ){
									if( (l==0 || l==7) && k!=7 && k!=0 ){
										
										if( position2.get( k+1, l ) != null &&  position2.get( k-1, l ) != null ){
											if( position2.get( k+1, l ).getPlayer() == myPlayer  &&  position2.get( k-1, l ).getPlayer() == myPlayer ){
												flag3 = true;
												
											}
										}
										
									}else{	
										if( ( k==0 || k==7  ) && l!=0 && l!=7  )
										if( position2.get( k, l+1 ) != null &&  position2.get( k, l-1 ) != null ){
											if( position2.get( k, l+1 ).getPlayer() == myPlayer  &&  position2.get( k, l-1 ).getPlayer() == myPlayer ){
												flag3 = true;
												
											}
										}
										
																												
									}								
								}
												
							}
						}
					}
					
					for( int k = 0 ;k < 8; k++ ){
						for( int l = 0; l < 8; l++ ){
							if( ( l==0 || l==7 || k==7 || k==0 ) && ( l != k ) && ( l+k != 7 ) ){
								if( position2.get( k, l ) == null ){
									if( l==0 || l==7  ){
										
										if( position2.get( k+1, l ) != null &&  position2.get( k-1, l ) != null ){
											if( position2.get( k+1, l ).getPlayer() == myPlayer  &&  position2.get( k-1, l ).getPlayer() == myPlayer ){
												flag3 = true;
												
											}
										}
										
									}else{	
										
										if( position2.get( k, l+1 ) != null &&  position2.get( k, l-1 ) != null ){
											if( position2.get( k, l+1 ).getPlayer() == myPlayer  &&  position2.get( k, l-1 ).getPlayer() == myPlayer ){
												flag3 = true;
												
											}
										}
										
																												
									}								
								}
												
							}
						}
					}
					
					if( ( ( !flag1 && !flag2 ) ||  ( flag1 && flag2 ) ) && ! flag3 )
							return move2;
				}
			}
		}
		
		for( int i = 2; i < 6; i += 3 ){
			for( int j = 2; j < 6; j += 3){
				IMove move = new Move( i, j, myPlayer );
				if( myGame.isMovePosible( myPosition, move ) ){
					return move;
				}
			}
		}
		
		for( int i = 2; i < 6; i++ ){
			for( int j = 2; j < 6; j++ ){
				IMove move = new Move( i, j, myPlayer );
				if( myGame.isMovePosible( myPosition, move ) ){
					return move;
				}
			}
		}
		
		for( int i = 1; i < 7; i++ ){
			for( int j = 1; j < 7; j++ ){
				if( ( i != j ) && ( i + j != 7 ) ){
					IMove move = new Move( i, j, myPlayer );
					if( myGame.isMovePosible( myPosition, move ) ){
						return move;
					}
				}				
			}
		}
		
		if( myPermit  ){
			while( true ){			
				int h = random.nextInt( myPosition.getHight() );
				int l = random.nextInt( myPosition.getWidth() );
				IMove move = new Move( h, l, myPlayer );
				if( myGame.isMovePosible( myPosition, move ) ){
					return move;
				}
			}
		}else{
			for( int h=0; h < 8; h++  ){
				for( int l = 0; l < 8; l++ ){
					IMove move = new Move( h, l, myPlayer );
					if( myGame.isMovePosible( myPosition, move ) ){
						return move;
					}
				}
			}			
		}
		
		return null;
		
	}

	public String getName() {
		return myName;
	}

	public void setGame(IGame game) {
		myGame = game;
		
	}

	public void setPosition(IPosition position) {
		
		myPosition = position;
	}

	public void setPlayer(int player) {
		
		myPlayer = player;
	}

	public void setRandom(boolean permit) {
		myPermit = permit;		
	}

}
