package test.org.lantsovanton;


import org.lantsovanton.abstraction.model.ICoordinate;
import org.lantsovanton.abstraction.model.IGame;
import org.lantsovanton.abstraction.model.IMove;
import org.lantsovanton.abstraction.model.IPosition;
import org.lantsovanton.abstraction.model.impl.Chip;
import org.lantsovanton.abstraction.model.impl.Coordinate;
import org.lantsovanton.abstraction.model.impl.Move;
import org.lantsovanton.abstraction.model.impl.Position;
import org.lantsovanton.games.reversi.Reversi;
import org.lantsovanton.movers.Pas;
//import org.lantsovanton.movers.CornersMove;

import junit.framework.TestCase;

public class ReversiTest extends TestCase {
	
	public void testposition(){
	
		IGame myGame = new Reversi();
		IPosition myPosition = myGame.initialPosition();
		IPosition idealposition = new Position(8,8,1);
		idealposition.set(3,3, new Chip(1) );  
		idealposition.set(4,4, new Chip(1) );  
		idealposition.set(4,3, new Chip(2) );  
		idealposition.set(3,4, new Chip(2) );  
		assertEquals( myPosition.toString(), idealposition.toString() );
	}
	
	public void testmovepossiblesimple(){
		IGame myGame = new Reversi();
		IPosition myPosition = myGame.initialPosition();
		
		// simple valid move
		IMove move = new Move(2,4,1);
		assertEquals( myGame.isMovePosible( myPosition, move), true );
	}	
	
	public void testmoveposssiblefieldempty(){
		IGame myGame = new Reversi();
		IPosition myPosition = myGame.initialPosition();
		// uncorrect move near field is empty
		IMove move = new Move(0,0,2);
		assertEquals( myGame.isMovePosible( myPosition, move), false );
		
	}	
	public void testmoveuncorrectplayer(){ 
		IGame myGame = new Reversi();
		IPosition myPosition = myGame.initialPosition();
		IMove move;
		// player uncorrect ( move correct for player of move )
		move = new Move(2,3,2);
		assertEquals( myGame.isMovePosible( myPosition, move), false );
	}
	public void testmoveplayer(){
		IGame myGame = new Reversi();
		IPosition myPosition = myGame.initialPosition();
		IMove move;
//		 player uncorrect ( move correct for player of position )
		move = new Move(2,4,2);
		assertEquals( myGame.isMovePosible( myPosition, move), false );
	}
	public void testmovepossible2(){
		IGame myGame = new Reversi();
		IPosition myPosition = myGame.initialPosition();
		IMove move;
		// uncorrect move : field not empty (self chip)
		myPosition.set(2,4,new Chip(1) );
		move = new Move(2,4,1);
	    assertEquals( myGame.isMovePosible( myPosition, move), false );
	}
	public void testmovepossible3(){
		IGame myGame = new Reversi();
		IPosition myPosition = myGame.initialPosition();
		IMove move;
	    // uncorrect move : field not empty (opposit chip)
		myPosition.set(2,4,new Chip(2) );
		move = new Move(2,4,1);
	    assertEquals( myGame.isMovePosible( myPosition, move), false );
	    myPosition = myGame.initialPosition();
	}
	public void testmovepossible4(){
			IGame myGame = new Reversi();
			IPosition myPosition = myGame.initialPosition();
			IMove move;    
	    //  uncorect move: out of board
	    for( int i = 1; i < 8; i++ ){
	    	for( int j = 1; j < 8; j++  ){
	    		myPosition.set(i,j, new Chip(2) );
	    	}
	    }
	    for( int l = 0; l < 8; l++ ){
	    	myPosition.set(0,l,new Chip(1) );
	    }
	    move = new Move(8,8,1);
	    assertEquals( myGame.isMovePosible( myPosition, move), false );
	}
	public void testmovepossible5(){
		IGame myGame = new Reversi();
		IPosition myPosition = myGame.initialPosition();
		IMove move;       
	    //diagonal eating
	    myPosition = new Position(8,8,1);
	    myPosition.set(1,1, new Chip(1) );
	    myPosition.set(2,2, new Chip(2) );
	    myPosition.set(3,3, new Chip(2) );
	    move = new Move(4,4,1);
	    assertEquals( myGame.isMovePosible( myPosition, move), true );
	}
	public void testmoveppossible6(){
			IGame myGame = new Reversi();
			IPosition myPosition = myGame.initialPosition();
			IMove move;	
	    // uncorrect type of move
	    move = new Pas(1);
	    assertEquals(myGame.isMovePosible( myPosition, move), false );
	    
	}
	
	public void testdomove(){
		
		//move in next test situation ( 4 situation ) reduce to finish of game
		//so number of player have not change 
		
		//gorizontal eating
		IGame game = new Reversi();
		IPosition position = game.initialPosition();
		IPosition positionideal = game.initialPosition();
		for( int i = 0; i < 8; i++ ){
		    	for( int j = 0; j < 8; j++  ){
		    		position.set(i,j, null );
		    	}
		}
		
		for( int l=0; l < 8; l++ ){
			position.set(0,l, new Chip(2) );
		}
		
		position.set(0,0, new Chip(1) );
		position.set(0,7, new Chip(1) );
		position.set(0,4, null );
		
		// set idealposition
		for( int i = 0; i < 8; i++ ){
	    	for( int j = 0; j < 8; j++  ){
	    		positionideal.set(i,j, null );
	    	}
       	}
	
    	for( int l=0; l < 8; l++ ){
	    	positionideal.set(0,l, new Chip(1) );
    	}
    	
		IMove move = new Move( 0, 4, 1 );
		game.doMove( position, move );
		assertEquals( position.toString(), positionideal.toString()  );
		
		//vertical eating
		position = game.initialPosition();
		positionideal = game.initialPosition();
		for( int i = 0; i < 8; i++ ){
	    	for( int j = 0; j < 8; j++  ){
	    		position.set( i, j, null );
	    		positionideal.set( i, j, null );
	    	}
	    }
		
		for( int h = 0; h < 8; h++ ){ 
			positionideal.set( h, 0, new Chip(1) );
			position.set( h, 0, new Chip( 2 ) );
		}
		
		position.set( 0, 0, new Chip( 1 ) );
		position.set( 7, 0, new Chip( 1 ) );
		position.set( 4, 0, null );
		move = new Move( 4, 0, 1 );
		game.doMove( position, move );
		assertEquals( position.toString(), positionideal.toString()  );
		
		//eating on main diagonal 
		position = game.initialPosition();
		positionideal = game.initialPosition();
		for( int i = 0; i < 8; i++ ){
	    	for( int j = 0; j < 8; j++  ){
	    		position.set( i, j, null );
	    		positionideal.set( i, j, null );
	    	}
	    }
		for( int d = 0; d < 8; d++ ){ 
			positionideal.set( d, d, new Chip(1) );
			position.set( d, d, new Chip( 2 ) );
		}
		position.set( 0, 0, new Chip( 1 ) );
		position.set( 7, 7, new Chip( 1 ) );
		position.set( 4, 4, null );
		move = new Move( 4, 4, 1 );
		game.doMove( position, move );
		assertEquals( position.toString(), positionideal.toString()  );
		
		//eating on side diagonal
		position = game.initialPosition();
		positionideal = game.initialPosition();
		for( int i = 0; i < 8; i++ ){
	    	for( int j = 0; j < 8; j++  ){
	    		position.set( i, j, null );
	    		positionideal.set( i, j, null );
	    	}
	    }
		for( int d = 0; d < 8; d++ ){ 
			positionideal.set(  d, 7 - d, new Chip(1) );
			position.set( 7 - d, d, new Chip( 2 ) );
		}
		position.set( 0, 7, new Chip( 1 ) );
		position.set( 7, 0, new Chip( 1 ) );
		position.set( 4, 3, null );
		move = new Move( 4, 3, 1 );
		game.doMove( position, move );
		assertEquals( position.toString(), positionideal.toString()  );
		
	}
	
	public void testchangeplayer(){ 
		
		// after this move other player can move
		IGame game = new Reversi();
		IPosition position = game.initialPosition();
		IMove move = new Move( 2, 4, 1 );
		game.doMove( position, move );
		assertEquals( position.getPlayer(), 2 );
		
		//other player cannot move but game not finish 
		position = game.initialPosition();		
		for( int i = 0; i < 8; i++ ){
	    	for( int j = 0; j < 8; j++  ){
	    		position.set( i, j, null );
	    		
	    	}
	    }
		
		for( int h = 0; h < 8; h++ ){ 
			position.set( h, 1, new Chip( 2 ) );
			position.set( h, 0, new Chip( 2 ) );
		}
		
		position.set( 0, 0, new Chip( 1 ) );
		position.set( 7, 0, new Chip( 1 ) );
		position.set( 4, 0, null );
		move = new Move( 4, 0, 1 );
		game.doMove( position, move );
		assertEquals( position.getPlayer(), 1  );
		
		//game finish (empty field exist)
		//expected that player would not changed
		position = game.initialPosition();
		
		for( int i = 0; i < 8; i++ ){
	    	for( int j = 0; j < 8; j++  ){
	    		position.set( i, j, null );
	    		
	    	}
	    }
		
		for( int h = 0; h < 8; h++ ){ 
			
			position.set( h, 0, new Chip( 2 ) );
		}
		
		position.set( 0, 0, new Chip( 1 ) );
		position.set( 7, 0, new Chip( 1 ) );
		position.set( 4, 0, null );
		move = new Move( 4, 0, 1 );
		game.doMove( position, move );
		assertEquals( position.getPlayer(), 1  );
        
		//game finish (empty field not exist)
		//expected that player would not changed
        position = game.initialPosition();
		
		for( int i = 0; i < 8; i++ ){
	    	for( int j = 0; j < 8; j++  ){
	    		position.set( i, j, new Chip( 2 ) );
	    		
	    	}
	    }	
		
		position.set( 0, 0, new Chip( 1 ) );
		position.set( 7, 0, new Chip( 1 ) );
		position.set( 4, 0, null );
		move = new Move( 4, 0, 1 );
		game.doMove( position, move );
		assertEquals( position.getPlayer(), 1  );		
	}
	
	public void testisgamefinish(){
		// empty board - finished game
		
		IGame game = new Reversi();
		IPosition position = new Position(8,8,1);
		assertEquals( game.isGameFinish( position ), true );
		
		// board  thick with chips completely
		game = new Reversi();
		position = new Position(8,8,1);		
		for( int i = 0; i < 8; i++ ){
	    	for( int j = 0; j < 8; j++  ){
	    		position.set( i, j, new Chip(1) );	    		
	    	}
	    }
		assertEquals( game.isGameFinish( position ), true );
		
		// empty field exist but move does not exist
		game = new Reversi();
		position = new Position(8,8,1);		
		for( int i = 0; i < 8; i++ ){
	    	for( int j = 0; j < 6; j++  ){
	    		position.set( i, j, new Chip(1) );	    		
	    	}
	    }
		assertEquals( game.isGameFinish( position ), true );
		// empty field exist but move does not exist another position
		game = new Reversi();
		position = new Position(8,8,1);		
		for( int i = 0; i < 8; i++ ){
	    	for( int j = 0; j < 8; j++  ){
	    		if( j < 4 )
	    		     position.set( i, j, new Chip( 1 ) );	 
	    		if( j > 4  )
	    			position.set( i, j, new Chip( 2 ) );	
	    	}
	    }
		assertEquals( game.isGameFinish( position ), true );
		
		// one of player can move but his number not equal myNextMovePlayer
		// in position
		// expected - true
		game = new Reversi();
		position = new Position(8,8,2);
		for( int h = 0; h < 8; h++ ){ 
			position.set( h, 0, new Chip( 1 ) );
			position.set( h, 1, new Chip( 2 ) );
		}
		assertEquals( game.isGameFinish( position ), true );
	}
	
	public void testgetname(){
		IGame game = new Reversi();
		assertEquals( game.getName(), "Reversi" );
	}
	
	public void testiswinner(){
		
		// in empty position return false
		IGame game = new Reversi();
		IPosition position = new Position(8,8,2);
		assertEquals( game.isWinner( position, 1 ), false );
		assertEquals( game.isWinner( position, 2 ), false );
		
		// if game not finished return false
		
		position = game.initialPosition();
		assertEquals( game.isWinner( position, 1 ), false );
		assertEquals( game.isWinner( position, 2 ), false );
		
        // one of player can move but his number not equal myNextMovePlayer
		// in position and one of player have chips than other
		// exepted - false
		
		game = new Reversi();
		position = new Position(8,8,2);
		for( int h = 0; h < 8; h++ ){ 
			position.set( h, 0, new Chip( 1 ) );
			position.set( h, 1, new Chip( 2 ) );
		}
		position.set( 0, 1, null );
		position.set( 7, 1, null );
		
		assertEquals( game.isWinner( position, 1 ), false );
		assertEquals( game.isWinner( position, 2 ), false );
		
		// board thick with chips of only one of player
		game = new Reversi();
		position = new Position(8,8,1);
		for( int i = 0; i < 8; i++ ){
	    	for( int j = 0; j < 8; j++  ){
	    		position.set( i, j, new Chip( 1 ) );
	    	}
	    }
		assertEquals( game.isWinner( position, 1 ), true );
		assertEquals( game.isWinner( position, 2 ), false );		
		
		// calculation chips
		game = new Reversi();
		position = new Position(8,8,2);
		for( int i = 0; i < 8; i++ ){
	    	for( int j = 0; j < 8; j++  ){
	    		if( j <= 2  )
	    		     position.set( i, j, new Chip( 1 ) );	 
	    		if( j > 2  )
	    			position.set( i, j, new Chip( 2 ) );	
	    	}
	    }
		assertEquals( game.isWinner( position, 1 ), false );
		assertEquals( game.isWinner( position, 2 ), true );	
		
	}
	
	public void testGetMove(){
		
		//move possible
		IGame game = new Reversi();
		IPosition position = game.initialPosition();
		ICoordinate coordinate = new Coordinate( 2, 4 );
		IMove move = new Move(2,4,1);
		assertEquals( move, game.getMove(position, coordinate) );
		
		//move unpossible
		game = new Reversi();
		position = game.initialPosition();
		coordinate = new Coordinate( 2, 5 );
		assertEquals( null, game.getMove(position, coordinate) );
	}
	
	
	
	

}
